/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

define([
    "dojo/_base/declare",
    "aps/xhr",
    "aps/FieldSet",
    "aps/nav/ViewPlugin",
    "./../components/BackupAdminActions",
    "./../components/getStorageOutput",
    "./../widgets/base/Output",
    "./../widgets/StatusTile",
    "./../../common",
    "./../../icons",
    "./../../messages"
], function (
    declare,
    xhr,
    FieldSet,
    ViewPlugin,
    BackupAdminActions,
    getStorageOutput,
    Output,
    StatusTile,
    common,
    icons,
    messages
) {
    var enduserStatuses = {
        INACTIVE: "0",
        ACTIVE: "1",
        IN_PROGRESS: "2",
        NOT_ACTIVATED: "3"
    };

    return declare(ViewPlugin, {
        _tile: null,
        _tenant: null,
        _currentUser: null,
        _selectedUser: null,
        _statistic: null,
        _backupAccountActions: null,

        _createResourceUsage: function (service) {
            var self = this,
                devicesCounters = common.getAvailableCountersForArea("devices_count_ccpv2"),
                storageCounters = common.getAvailableCountersForArea("storage_usage_only"),
                devicesTotal = devicesCounters.reduce(function (total, counter) {
                    return total + (service[counter.name] ? service[counter.name].usage : 0);
                }, 0),
                storageUsage = {},
                fs = new FieldSet({}),
                gridSize = "md-6 xs-12";

            storageCounters.forEach(function (counter) {
                if (service[counter.name]) {
                    storageUsage = service[counter.name];
                }
            });

            fs.addChild(getStorageOutput({
                label: _('Cloud Storage', {}, {}, self),
                limitedLegend: _("${usage} ${usageUnit} of ${limit} ${limitUnit}", {}, {}, self),
                unlimitedLegend: "${usage} ${usageUnit} used",
                usage: storageUsage.usage || 0,
                limit: storageUsage.limit || 0,
                gridSize: gridSize
            }, {}, {}, self));
            fs.addChild(new Output({
                label: _('Devices', {}, {}, self),
                content: _("${value} Devices Total", {}, {}, self),
                value: devicesTotal,
                gridSize: gridSize
            }));
            fs.addChild(new Output({
                label: _('Login', {}, {}, self),
                value: service.endUserLogin,
                gridSize: "md-12 xs-12"
            }));

            return fs;
        },

        _createTile: function (service) {
            var dropDownButton,
                menuItems,
                self = this,
                statusInfo = {
                    updating: {"label": _("UPDATING", {}, {}, self), "type": "default", isLoad: true},
                    deleting: {"label": _("DELETING", {}, {}, self), "type": "danger", isLoad: true},
                    disabled: {"label": _("DISABLED", {}, {}, self), "type": "default", isLoad: true},
                    notActivated: {"label": _("NOT ACTIVATED", {}, {}, self), "type": "warning"},
                    activating: {"label": _("ACTIVATING", {}, {}, self), "type": "warning", isLoad: true, useIcon: false},
                    activated: {"label": _("ACTIVATED", {}, {}, self), "type": "success"}
                };

            self._tile = new StatusTile({
                statusInfo: statusInfo,
                title: _("Cyber Protection", {}, {}, self),
                fontColor: "#ffffff",
                backgroundColor: "#163584",
                iconName: self.buildStaticURL(icons.ACRONIS_BACKUP_TILE_150)
            });

            if (!service) {
                self._tile.addChild(new Output({value: _("Service is not assigned yet", {}, {}, self)}));
                self._tile.set({
                    status: null,
                    buttons: [{
                        iconClass: "fa-check",
                        autoBusy: false,
                        title: _("Assign Service", {}, {}, self),
                        onClick: function () {
                            self.onAssign();
                        }
                    }]
                });
                return;
            }

            var separatorMenuItem = {
                    type: "separator"
                },
                activateMenuItem = {
                    iconClass: "fa-magic",
                    type: "warning",
                    title: _("Activate", {}, {}, self),
                    onClick: function () {
                        self.onActivate();
                    }
                },
                configureMenuItem = {
                    iconClass: "fa-pencil",
                    title: _("Change Settings", {}, {}, self),
                    onClick: function () {
                        self.onConfigure();
                    }
                },
                unassignMenuItem = {
                    iconClass: "fa-times",
                    title: _("Unassign Service", {}, {}, self),
                    onClick: function () {
                        self.onUnassign();
                    }
                },
                loginToImpersonatedBackupConsole = {
                    iconClass: "fa-external-link",
                    title: _("Log into Backup Console", {}, {}, self),
                    onClick: function () {
                        self.onLogIntoConsole();
                    }
                },
                passwordMenuItem = {
                    iconClass: "fa-key",
                    title: _("Change Password", {}, {}, self),
                    onClick: function () {
                        self.onChangePassword();
                    }
                },
                enableMenuItem = {
                    iconClass: "fa-times",
                    title: _("Enable", {}, {}, self),
                    onClick: function () {
                        self.onEnable();
                    }
                },
                disableMenuItem = {
                    iconClass: "fa-times",
                    title: _("Disable", {}, {}, self),
                    onClick: function () {
                        self.onDisable();
                    }
                };

            switch (service.aps.status) {
                case "aps:proto":
                case "aps:provisioning":
                    configureMenuItem.disabled = true;
                    unassignMenuItem.disabled = true;
                    self._tile.set({
                        status: 'updating',
                        buttons: [{
                            iconClass: "fa-cog",
                            title: _("Actions", {}, {}, self),
                            items: [configureMenuItem, separatorMenuItem, unassignMenuItem]
                        }]
                    });
                    self._tile.addChild(new Output({value: _("Please wait...", {}, {}, self)}));
                    break;
                case "aps:unprovisioning":
                case "aps:deleted":
                    self._tile.set({
                        status: 'deleting',
                        buttons: []
                    });
                    self._tile.addChild(new Output({value: _("Please wait...", {}, {}, self)}));
                    break;
                case "aps:ready":
                    if (service.userstatus == enduserStatuses.NOT_ACTIVATED) {
                        self._tile.set({
                            status: 'notActivated',
                            buttons: [{
                                type: "warning",
                                iconClass: "fa-bolt",
                                title: _("Actions", {}, {}, self),
                                items: [activateMenuItem, separatorMenuItem, unassignMenuItem]
                            }],
                            onClick: function () {
                                self.onActivate();
                            }
                        });
                    }
                    else if (service.userstatus == enduserStatuses.ACTIVE) {
                        menuItems = [loginToImpersonatedBackupConsole, separatorMenuItem, configureMenuItem, separatorMenuItem];
                        if (self._currentUser.aps.id === self._selectedUser.aps.id) {
                            menuItems = menuItems.concat([passwordMenuItem, separatorMenuItem]);
                        }

                        if (service.servicestatus == common.STATUS.ENABLED) {
                            menuItems.push(disableMenuItem);
                        }
                        menuItems.push(unassignMenuItem);
                        dropDownButton = {
                            iconClass: "fa-cog",
                            title: _("Actions", {}, {}, self),
                            items: menuItems
                        };

                        self._tile.set({
                            status: null,
                            buttons: [dropDownButton],
                            onClick: function () {
                                self.onConfigure();
                            }
                        });
                    }
                    else if (service.userstatus == enduserStatuses.IN_PROGRESS) {
                        self._tile.set({
                            status: 'inProgress'
                        });
                    }
                    else {
                        menuItems = [enableMenuItem, unassignMenuItem];
                        dropDownButton = {
                            iconClass: "fa-cog",
                            title: _("Actions", {}, {}, self),
                            items: menuItems
                        };
                        self._tile.set({
                            status: 'disabled',
                            buttons: [dropDownButton]
                        });
                    }
                    var data = self._statistic && self._statistic[service.userId] ? self._statistic[service.userId] : service;
                    data.endUserLogin = service.endUserLogin;
                    self._tile.addChild(self._createResourceUsage(data));
                    break;
            }
        },

        onAssign: function () {
            var self = this;

            self._tile.set('isBusy', true);
            self._backupAccountActions.assign().then(function (created) {
                if (created) {
                    self._tile.set({
                        status: 'updating',
                        buttons: []
                    });
                    self._tile.removeAll();
                    self._tile.addChild(new Output({value: _("Please wait...", {}, {}, self)}));
                    return;
                }
                self._tile.set('isBusy', false);
            }, messages.displayError);
        },

        onConfigure: function () {
            var self = this;

            self._tile.set('isBusy', true);
            self._backupAccountActions.configure().then(function (created) {
                if (created) {
                    self._tile.set({
                        status: 'updating',
                        buttons: []
                    });
                    self._tile.removeAll();
                    self._tile.addChild(new Output({value: _("Please wait...", {}, {}, self)}));
                    return;
                }
                self._tile.set('isBusy', false);
            }, messages.displayError);
        },

        onUnassign: function () {
            var self = this;

            self._tile.set('isBusy', true);
            self._backupAccountActions.unassign().then(function (created) {
                if (created) {
                    self._tile.set({
                        status: 'updating',
                        buttons: []
                    });
                    self._tile.removeAll();
                    self._tile.addChild(new Output({value: _("Please wait...", {}, {}, self)}));
                    return;
                }
                self._tile.set('isBusy', false);
            }, messages.displayError);
        },

        onActivate: function () {
            var self = this;

            self._tile.set('isBusy', true);
            self._backupAccountActions.activate().then(function (created) {
                if (created) {
                    self._tile.set({
                        status: 'updating',
                        buttons: []
                    });
                    self._tile.removeAll();
                    self._tile.addChild(new Output({value: _("Please wait...", {}, {}, self)}));
                    return;
                }
                self._tile.set('isBusy', false);
            }, messages.displayError);
        },

        onChangePassword: function () {
            var self = this;

            self._tile.set('isBusy', true);
            self._backupAccountActions.changePassword().then(function (created) {
                if (created) {
                    self._tile.set({
                        status: 'updating',
                        buttons: []
                    });
                    self._tile.removeAll();
                    self._tile.addChild(new Output({value: _("Please wait...", {}, {}, self)}));
                    return;
                }
                self._tile.set('isBusy', false);
            }, messages.displayError);
        },

        onLogIntoConsole: function () {
            var self = this;

            var w = window.open('', '_blank');
            self._backupAccountActions.loginToConsole().then(function (uri) {
                w.location.href = uri;
            }, function (error) {
                w.close();
                messages.displayError(error);
            });
        },

        onEnable: function () {
            var self = this;
            self._tile.set('isBusy', true);
            self._backupAccountActions.enable().then(function () {
                self._tile.set({
                    status: 'updating',
                    buttons: []
                });
                self._tile.removeAll();
                self._tile.addChild(new Output({value: _("Please wait...", {}, {}, self)}));
            }, messages.displayError);
        },

        onDisable: function () {
            var self = this;
            self._tile.set('isBusy', true);
            self._backupAccountActions.disable().then(function () {
                self._tile.set({
                    status: 'updating',
                    buttons: []
                });
                self._tile.removeAll();
                self._tile.addChild(new Output({value: _("Please wait...", {}, {}, self)}));
            }, messages.displayError);
        },

        init: function (mediator) {
            var self = this;
            mediator.setServiceData = function (data) {
                var services = data.services.filter(function (service) {
                        return service.aps.subscription === self._tenant.aps.subscription;
                    }),
                    service = services.length ? services.pop() : null;
                self._createTile(service);
                return self._tile;
            };
        },

        onContext: function (context) {
            var self = this;
            self._tenant = context.vars.tenant;
            self._currentUser = aps.context.user;
            self._selectedUser = aps.context.vars.selectedUser;
            self._backupAccountActions = new BackupAdminActions(self._selectedUser, self._tenant, self);
            xhr.get("/aps/2/resources/" + self._tenant.aps.id + "/updateStatistics", {sync: true}).then(function (response) {
                self._statistic = response.userUsage;
            });
        }
    });
});
