/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

define([
    'dojo/_base/declare',
    'dojo/Deferred',
    'dojo/promise/all',
    'aps/_View',
    'aps/xhr',
    'aps/nav/ViewPluginManager',
    './../components/BackupAdminActions',
    './../components/getDownloadTile',
    './../components/getUserResourcesTile',
    './../components/getTenantResourcesTile',
    './../widgets/UserGrid',
    './../widgets/MobileUserGrid',
    './../../common',
    './../../messages'
], function (
    declare,
    Deferred,
    all,
    _View,
    xhr,
    ViewPluginManager,
    BackupAdminActions,
    getDownloadTile,
    getUserResourcesTile,
    getTenantResourcesTile,
    UserGrid,
    MobileUserGrid,
    common,
    messages
) {

    function getBackupUserStatus(backupUser) {
        if (backupUser === null) {
            return 'notAssigned';
        }

        var enduserStatuses = {
            INACTIVE: '0',
            ACTIVE: '1',
            IN_PROGRESS: '2',
            NOT_ACTIVATED: '3'
        };

        switch (backupUser.aps.status) {
            case 'aps:proto':
            case 'aps:provisioning':
                return 'updating';
            case 'aps:unprovisioning':
            case 'aps:deleted':
                return 'deleting';
            case 'aps:ready':

                var statusName = 'disabled';
                if (backupUser.userstatus == enduserStatuses.NOT_ACTIVATED) {
                    statusName = 'notActivated';
                }
                else if (backupUser.userstatus == enduserStatuses.IN_PROGRESS) {
                    statusName = 'inProgress';
                }
                else if (backupUser.userstatus == enduserStatuses.ACTIVE) {
                    statusName = 'activated';
                }
                return statusName;
        }

        return true;
    }

    function getBackupUserStatistic(backupUser, backupUsersStatistic) {
        if (backupUser && backupUsersStatistic[backupUser.userId] !== undefined) {
            return backupUsersStatistic[backupUser.userId];
        }

        return null;
    }

    function findBackupUserByPaUserId(backupUsers, paUserId) {
        var user = null;

        backupUsers.some(function (backupUser) {
            if (backupUser.serviceUserId === paUserId) {
                user = backupUser;
                return true;
            }
            return false;
        });

        return user;
    }

    function joinPaUsersWithBackupStatistic(paUsers, backupUsers, backupUsersStatistic) {
        return paUsers.map(function (paUser) {
            var backupUser = findBackupUserByPaUserId(backupUsers, paUser.aps.id);
            paUser.backupUser = backupUser;
            paUser.backupUserStatus = getBackupUserStatus(backupUser);
            paUser.backupUserStatistic = getBackupUserStatistic(backupUser, backupUsersStatistic);
            return paUser;
        });
    }

    return declare(_View, {
        _backupTilesId: null,

        _administrationTilesId: null,

        _getSubscriptionMediator: function () {
            var managerSubscription = new ViewPluginManager('subscriptionPlugin');
            return managerSubscription.init().then(function () {
                return managerSubscription.get('mediator');
            });
        },

        _getUsersGrid: (function () {
            var _grid, gridOptions = {noDataText: _('No users available')};
            return function () {
                if (typeof _grid === 'undefined') {
                    gridOptions.id = this.genId('users');
                    _grid = aps.context.visibleMode === 'xs' ? new MobileUserGrid(gridOptions) : new UserGrid(gridOptions);
                    _grid.placeAt(this.domNode);
                }
                return _grid;
            };
        })(),

        init: function () {
            var self = this;

            self._backupTilesId = self.genId('backup_tiles');
            self._administrationTilesId = self.genId('administration_tiles');
            return [
                ['aps/FieldSet', {showLabels: false}, [
                    ['aps/Output', {
                        id: self.genId('description'),
                        value: _('Welcome to Acronis Cyber Cloud, a unique integration of backup with full-stack next-generation anti-malware protection and comprehensive endpoint management tools. This synergy eliminates complexity and improves security so you can have better protection for your data.')
                    }]
                ]],
                ['aps/Tiles', {
                    id: self._backupTilesId,
                    title: ''
                }],
                ['aps/Tiles', {
                    id: self._administrationTilesId,
                    title: _('Administration')
                }]
            ];
        },
        onHide: function () {
            messages.removeAll();
        },
        onContext: function () {
            var self = this,
                user = aps.context.user,
                users = aps.context.vars.users,
                tenant = aps.context.vars.tenant;

            /**
             * Determines if service is assigned (also checks that provided backupAccount is not empty value)
             * @param backupAccount
             * @returns {*|boolean}
             */
            function isServiceAssigned(backupAccount) {
                return backupAccount && backupAccount.aps.status === 'aps:ready' && backupAccount.userstatus === '1';
            }

            /**
             * Gets us a list of agents to download by backupAccount
             * Fails over to returning empty object provided backend issues an error.
             * @param backupAccount
             * @returns {*}
             */
            function fetchAgentsList(backupAccount) {
                if (isServiceAssigned(backupAccount)) {
                    var agentsURI = '/aps/2/resources/' + backupAccount.aps.id + '/agents?language=' + common.getLanguage(aps.context._locale);
                    return xhr.get(agentsURI);
                } else {
                    var deffered = new Deferred();
                    deffered.resolve(null);

                    return deffered;
                }
            }

            /**
             * Download Backup Agent(s) Widget
             *
             * @param backupAccount
             * @param backupAccountStatistic
             * @param countersList
             */
             function drawBackupAgentsWidget(backupAccount, backupAccountStatistic, countersList) {
                var backupTiles = self.byId(self._backupTilesId),
                    backupAccountActions = new BackupAdminActions(user, tenant);

                backupTiles.removeAll();
                backupTiles.addChild(getUserResourcesTile(backupAccount, backupAccountStatistic, backupAccountActions, tenant, countersList, user));

                if (backupAccount && backupAccount.userstatus === common.USER_STATUS.ACTIVE) {
                    backupTiles.addChild(getDownloadTile('nextcp-ccp-download', backupAccount.aps.id));
                }
            }

            /**
             * Statistics Widget
             *
             * @param backupStatistic
             * @param billingResources
             * @param countersList
             */
            var drawStatistics = function(backupStatistic, billingResources, countersList) {
                var subscriptionId = tenant.subscription.subscriptionId,
                    administrationTiles = self.byId(self._administrationTilesId);

                administrationTiles.removeAll();
                administrationTiles.addChild(getTenantResourcesTile(
                    backupStatistic.tenant,
                    subscriptionId,
                    tenant.subscription.trial,
                    (billingResources.length > 0),
                    countersList
                ));
            };
            all({
                backupUsers: xhr.get('/aps/2/resources/' + tenant.aps.id + '/users'),
                backupStatistic: xhr.get('/aps/2/resources/' + tenant.aps.id + '/updateStatistics'),
                billingResources: aps.biz.getResourcesInfo({filter:[{apsType: common.TYPES.TENANT}], subscriptionId:tenant.aps.subscription})
            }).then(function (response) {
                var usersWithStatistic = joinPaUsersWithBackupStatistic(users, response.backupUsers, response.backupStatistic.userUsage),
                    backupAccount = findBackupUserByPaUserId(response.backupUsers, user.aps.id);
                var draw = function(backupAccountStatistic) {
                    var countersListBackupUser = common.getAvailableCountersForArea('user_quotas'),
                        countersListResources = common.getAvailableCountersForArea('resources_tile_ccpv2');
                    drawBackupAgentsWidget(backupAccount, backupAccountStatistic, countersListBackupUser);
                    drawStatistics(response.backupStatistic, response.billingResources, countersListResources);

                    var grid = self._getUsersGrid();
                    grid.set('users', usersWithStatistic);

                    aps.apsc.hideLoading();
                }
                if (backupAccount) {
                    common.getBackupUserData(backupAccount).then(function (backupUser) {
                        backupAccount = backupUser;
                        draw(backupAccount ? response.backupStatistic.userUsage[backupAccount.userId] : null);
                    });
                } else {
                    draw(null);
                }
            });
        }
    });
});

