/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "aps/Memory",
    "aps/Output",
    "aps/xhr",
    "dijit/registry",
    "dojo/promise/all",
    "dojox/mvc/StatefulArray",
    "./js/common.js",
    "./js/messages.js",
    "./js/wizardData.js",
    "./js/tags.js"
], function (
    load,
    Memory,
    Output,
    xhr,
    registry,
    all,
    StatefulArray,
    common,
    messages,
    wizardData,
    tags
) {
    var globals = aps.context.vars.globals,
        reseller = wizardData.getWizardData(common.getRelatedViewId('configuration'));

    function goToList() {
        aps.apsc.gotoView(common.getRelatedViewId('list'));
    }

    if (!reseller) {
        goToList();
    }

    function fetchDiscountById(discountId) {
        return xhr.get("/aps/2/resources/" + globals.aps.id + "/discount/" + discountId);
    }

    function verifyAllowingDiscountForCurrentReseller(discount) {
        if (discount.ownerAccountId != reseller.accountId) {
            throw _('Discount does not belong to this reseller.');
        }
        if (!discount.isAllowedForFirstLevelReseller) {
            throw _('Discount is not allowed to use.');
        }
    }

    var columns = [
        {
            field: "id",
            name: _("ID")
        },
        {
            field: "name",
            name: _("Name")
        },
        {
            field: "ownerAccountId",
            name: _("Owner ID")
        },
        {
            name: _("Type"),
            sortable: false,
            renderCell: function () {
                return _("Discount By Account/Customer Class");
            }
        },
        {
            name: _("Status"),
            sortable: false,
            escapeHTML: false,
            renderCell: function (discount) {
                return discount.status ?
                    tags.img('images/statuses/yes16.gif') + ' ' + _("Active"):
                    tags.img('images/statuses/no16.gif') + ' ' + _("Inactive");
            }
        },
        {
            name: _("Discount Rate (%)"),
            sortable: false,
            escapeHTML: false,
            renderCell: function (discount) {
                return Number(discount.rate).toFixed(2) + _("%");
            }
        },
        {
            name: _("For All Plans"),
            sortable: false,
            escapeHTML: false,
            renderCell: function (discount) {
                return discount.isForAllPlans ?
                    tags.img("images/statuses/yes16.gif") + _("Yes") :
                    tags.img("images/statuses/no16.gif") + _("No");
            }
        },
        {
            name: _("For All Resources"),
            sortable: false,
            escapeHTML: false,
            renderCell: function (discount) {
                return discount.isForAllResources ?
                    tags.img("images/statuses/yes16.gif") + _("Yes") :
                    tags.img("images/statuses/no16.gif") + _("No");
            }
        }
    ];

    var infoHtml = tags.div([
        _("Only those Discounts that are created under the proper L1 reseller, are active, have \"Discount By Account/Customer Class\" type, have 100% Discount Rate and \"For all resources\" and \"For all service plans\" options disabled are available for selection."),
        '<br/>',
        _("Please take into account that already provisioned discounts should be un-assigned manually."),
        '<br/>',
        '<br/>',
        _("Reseller: __id__ __name__", {id: reseller.accountId, name: reseller.name}),
        '<br/>',
        '<br/>',
        _("Discount ID:"),
        '<br/>'
    ].join(""));

    var widgets = ["aps/PageContainer", {id: "apsPageContainer", "class": ""}, [
        ["aps/Container", {
            id: common.genId('container'),
            title: _("Configure a Discount")
        }, [
            ["aps/Output", {
                id: common.genId('output'),
                escapeHTML: false,
                value: infoHtml
            }],
            ["aps/TextBox", {
                id: 'discountIdInput',
                value: reseller ? reseller.discountId : '',
                pattern: "^[0-9]+$",
                invalidMessage: _('Discount ID should be a number.'),
                required: true,
                trim: true,
                missingMessage: _('Discount ID is required')
            }],
            ["aps/Button", {
                id: "searchDiscount",
                title: _("Search"),
                isDefault: true,
                onClick: function () {
                    var self = this;

                    var page = registry.byId('apsPageContainer');
                    if (!page.validate()) {
                        self.cancel();
                        return;
                    }

                    var discountsGrid = registry.byId("discountsGrid"),
                        discountId = registry.byId("discountIdInput").value;

                    messages.removeAll();

                    fetchDiscountById(discountId).then(function (discount) {
                        discountsGrid.set("store", new Memory({data: [discount], idProperty: "id"}));
                        discountsGrid.refresh();
                        discountsGrid.set('visible', true);

                        try {
                            verifyAllowingDiscountForCurrentReseller(discount);
                        } catch (e) {
                            messages.displayMessage(e, 'warning');
                        }

                        self.cancel();
                    }, function (error) {
                        messages.displayError(error);
                        discountsGrid.set('visible', false);
                        self.cancel();
                    });
                }
            }],
            ["aps/Grid", {
                selectionMode: false,
                id: "discountsGrid",
                showPaging: false,
                pageSizeOptions: "GRID_UNLIMITED",
                columns: columns
            }]
        ]]
    ]];

    load(widgets).then(function () {
        if (reseller.discountId) {
            registry.byId("searchDiscount").emit('click');
        }

        aps.app.onSubmit = function () {
            var page = registry.byId('apsPageContainer');
            if (!page.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }

            messages.removeAll();

            var discountId = registry.byId("discountIdInput").value;
            fetchDiscountById(discountId).then(function (discount) {
                try {
                    verifyAllowingDiscountForCurrentReseller(discount);
                } catch (e) {
                    messages.displayMessage(e, 'error');
                    aps.apsc.cancelProcessing();
                    return;
                }

                xhr("/aps/2/resources/" + globals.aps.id + "/reseller/" + reseller.accountId + "/discount", {
                    method: "PUT",
                    headers: {"Content-Type": "application/json"},
                    data: JSON.stringify({discountId: discount.id})
                }).then(goToList, messages.displayError);

            }, messages.displayError);
        };
    }, messages.displayError);

    aps.app.onCancel = goToList;
});
