/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "aps/Memory",
    "aps/ResourceStore",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/getStateful",
    "./js/common.js",
    "./js/messages.js"
], function (
    load,
    Memory,
    Store,
    registry,
    at,
    getPlainValue,
    getStateful,
    common,
    messages
) {

    function isEmptyCommitment(obj) {
        if (obj == null) return true;
        if (typeof obj !== "object") return true;
        return (!obj.currency && !obj.value);
    }

    function isUniqueCurrency(commitments) {
        var distinct = {};
        return !commitments.some(function (commitment) {
            var currency = String(commitment.currency).toUpperCase();
            if (distinct[currency]) {
                return true;
            }
            distinct[currency] = true;
            return false;
        });
    }

    var tier = aps.context.vars.tier,
        resourcesStore = new Store({
            target: "/aps/2/resources/"
        }),
        tierModel = getStateful({
            aps: {id: tier.aps.id},
            name: tier.name,
            minimalCommitments: tier.minimalCommitments
        }),
        newMinimalCommitments = getStateful([]),
        isCurrencyDisabled = tier.level < 0;

    var widgets = [
        ["aps/FieldSet", {id: common.genId('fieldset'), title: _("General")}, [
            ["aps/TextBox", {
                id: "name",
                label: _("Minimum commitment tier name"),
                value: at(tierModel, "name"),
                required: true,
                trim: true,
                hint: _("The resource name will not be automatically changed if you change the minimum commitment tier name.")
            }],
            ["aps/Output", {
                id: "resourceType",
                label: _("Resource"),
                value: tier.resourceTypeId
            }],
            ["aps/Output", {
                id: "level",
                label: _("Level"),
                value: common.getMinimalCommitmentLevelName(tier.level)
            }]
        ]]
    ];
    if (!isCurrencyDisabled) {
        widgets = widgets.concat([
            ["aps/WidgetList", {
                id: common.genId('widgetlist'),
                children: at(tierModel, "minimalCommitments"),
                showAddRemove: false
            }, [
                ["aps/FieldSet", {title: _("Minimum commitment")}, [
                    ["aps/Output", {
                        id: common.genId('output'),
                        label: _("Currency"),
                        value: at("rel:", "currency")
                    }],
                    ["aps/TextBox", {
                        id: common.genId('textbox'),
                        label: _("Value"),
                        value: at("rel:", "value"),
                        required: true,
                        pattern: '^(0|[1-9]\\d*)(\\.\\d{1,2})?$',
                        hint: _("For example, 123.45")
                    }]
                ]]
            ]],
            ["aps/WidgetList", {
                id: "newMinimalCommitments",
                children: newMinimalCommitments
            }, [
                ["aps/FieldSet", {
                    id: common.genId('fieldset'),
                    title: _("Minimum commitment")
                }, [
                    ["aps/TextBox", {
                        id: common.genId('textbox'),
                        label: _("Currency"),
                        value: at("rel:", "currency").transform({
                            format: function (value) {
                                return String(value).toUpperCase();
                            },
                            parse: function (value) {
                                return String(value).toUpperCase();
                            }
                        }),
                        required: true,
                        pattern: '^[A-Za-z]{3}$',
                        hint: _("For example, USD")
                    }],
                    ["aps/TextBox", {
                        id: common.genId('textbox'),
                        label: _("Value"),
                        value: at("rel:", "value"),
                        required: true,
                        pattern: '^(0|[1-9]\\d*)(\\.\\d{1,2})?$',
                        hint: _("For example, 123.45")
                    }]
                ]]
            ]]
        ]);
    }

    load(["aps/PageContainer", {id: "apsPageContainer"}, widgets]).then(function () {
        aps.app.onSubmit = function () {
            var widgetList,
                newMinimalCommitmentsData = getPlainValue(newMinimalCommitments);
            if (!isCurrencyDisabled && newMinimalCommitmentsData.length == 1 && isEmptyCommitment(newMinimalCommitmentsData[0])) {
                newMinimalCommitmentsData = [];
                widgetList = registry.byId("newMinimalCommitments");
                widgetList.set('disabled', true);
            }
            var page = registry.byId("apsPageContainer");
            if (!page.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }
            !isCurrencyDisabled && widgetList && widgetList.set('disabled', false);

            var tierData = getPlainValue(tierModel);
            tierData.minimalCommitments = isCurrencyDisabled ? [] : tierData.minimalCommitments.concat(newMinimalCommitmentsData);

            if (!isUniqueCurrency(tierData.minimalCommitments)) {
                messages.displayWarning(_('Please remove values of minimum commitment in the same currency.'), false);
                aps.apsc.cancelProcessing();
                return;
            }
            resourcesStore.put(tierData).then(function () {
                aps.apsc.gotoView(common.getRelatedViewId('view'), tier.aps.id);
            }, messages.displayError);
        };
    }, messages.displayError);

    aps.app.onCancel = function () {
        aps.apsc.gotoView(common.getRelatedViewId('view'), tier.aps.id);
    };
});
