/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

/*
 * Methods used for calculating quotas and render quota editing controls.
 */
define([
        'dojo/_base/lang',
        'aps/TextBox',
        './js/common.js',
        './js/uomConverter.js'
    ],
    function (
        lang,
        TextBox,
        common,
        uomConverter
    ) {
        function getQuotaControls(user, tenant, countersToDisplay, quotasStatistics, setToMax, userQuotaMode) {
            var controls = {};

            Object.values(countersToDisplay).forEach(function (counter) {
                var counterName = counter.name;
                if (tenant[counterName] && tenant[counterName].limit) {

                    if (!quotasStatistics.hasOwnProperty(counterName)) {
                        return;
                    }
                    var userOfferingItem = user && common.getUserOfferingItem(user, counterName),
                        controlConfig = {
                            id: 'user' + counterName,
                            label: formatStorageLabel(counter.label, tenant[counterName].limit),
                            quotaStatistic: quotasStatistics[counterName],
                            allocatedToUser: userOfferingItem && userOfferingItem.limit ? userOfferingItem.limit : 0,
                            isStorage: counter.legendSchema === 'storage',
                            setToMax: setToMax,
                            userQuotaMode: userQuotaMode,
                            isForNewUser: !user || user.changed
                        };

                    try {
                        var control = createTextBox(controlConfig);
                        if (control) {
                            controls[counterName] = control;
                        }
                    } catch (e) {
                        console.log('Unable to register control for' + counterName + 'most probably it was already registered. Original exception:' + e.message);
                    }
                }
            });

            return controls;
        }

        function createTextBox(config) {

            if (config.quotaStatistic) {
                var allocatedTotal = config.quotaStatistic.allocatedTotal == common.QUOTA_UNLIMITED_VALUE ?
                    common.QUOTA_UNLIMITED_VALUE : (
                        config.isStorage ?
                            Math.ceil(config.quotaStatistic.allocatedTotal / common.BYTES_IN_GB) :
                            config.quotaStatistic.allocatedTotal
                    );
            }

            var allocatedToUser = config.allocatedToUser == common.QUOTA_UNLIMITED_VALUE ?
                common.QUOTA_UNLIMITED_VALUE : (
                    config.isStorage ?
                        Math.ceil(config.allocatedToUser / common.BYTES_IN_GB) :
                        config.allocatedToUser
                );

            var availableToAllocate = common.QUOTA_UNLIMITED_VALUE,
                limit = common.QUOTA_UNLIMITED_VALUE;

            if (config.quotaStatistic && config.quotaStatistic.limit != common.QUOTA_UNLIMITED_VALUE && config.userQuotaMode == common.USER_QUOTA.STRICT) {
                limit = config.isStorage ?
                    Math.floor(config.quotaStatistic.limit / common.BYTES_IN_GB) :
                    config.quotaStatistic.limit;

                var removeAllocated = config.isForNewUser ? 0 : allocatedToUser;
                availableToAllocate = allocatedTotal != common.QUOTA_UNLIMITED_VALUE ?
                    Math.max(limit - (allocatedTotal - removeAllocated), 0) : 0;
            }

            if (config.isStorage) {
                config.label = formatStorageLabel(config.label, config.quotaStatistic.limit);
            }

            var isAvailableToAllocateUnlimited = availableToAllocate == common.QUOTA_UNLIMITED_VALUE
                || config.userQuotaMode === common.USER_QUOTA.OFF;

            if (!isAvailableToAllocateUnlimited && limit > 0 && config.allocatedToUser == common.QUOTA_UNLIMITED_VALUE) {
                allocatedToUser = 0;
            }

            function getQuotaHint() {
                var dataForText = {
                    user_alloc: allocatedToUser < 0 ? _('Unlimited') : allocatedToUser,
                    total_alloc: allocatedTotal < 0 ? _('Unlimited') : allocatedTotal,
                    limit: limit < 0 ? _('Unlimited') : limit,
                    available_alloc: availableToAllocate < 0 ? _('Unlimited') : availableToAllocate
                };

                if (isAvailableToAllocateUnlimited) {
                    if (config.isForNewUser || allocatedToUser == common.QUOTA_UNLIMITED_VALUE) {
                        return _('Resource is not limited.');
                    }
                    return config.isStorage ?
                        _('__user_alloc__ GB currently allocated to this user.', dataForText) :
                        _('__user_alloc__ currently allocated to this user.', dataForText);
                }

                if (config.isForNewUser || !allocatedToUser) {
                    return config.isStorage ?
                        _('__total_alloc__ GB of __limit__ GB allocated in total, __available_alloc__ GB available to the user.', dataForText) :
                        _('__total_alloc__ of __limit__ allocated in total, __available_alloc__ available to the user.', dataForText);
                }

                return config.isStorage ?
                    _('__user_alloc__ GB currently allocated to this user. __total_alloc__ GB of __limit__ GB allocated in total, __available_alloc__ GB available to the user.', dataForText) :
                    _('__user_alloc__ currently allocated to this user. __total_alloc__ of __limit__ allocated in total, __available_alloc__ available to the user.', dataForText);
            }

            var constraints = {
                    min: 0
                },
                invalidMessage = _('The value must be between __min__ and __max__.', constraints);

            if (availableToAllocate !== common.QUOTA_UNLIMITED_VALUE) {
                constraints.max = availableToAllocate;
                invalidMessage = _('The value must not exceed {0}').replace('{0}', constraints.max);
            }

            var inputValue = !allocatedToUser || allocatedToUser < 0 ? availableToAllocate : allocatedToUser;
            return new TextBox({
                id: config.id,
                label: config.label,
                placeHolder: isAvailableToAllocateUnlimited ? _('Unlimited') : '',
                required: !isAvailableToAllocateUnlimited,
                value: !isAvailableToAllocateUnlimited && config.setToMax || availableToAllocate === 0 ? inputValue : '',
                pattern: config.pattern || '^[0-9]+$',
                constraints: constraints,
                hint: getQuotaHint() + (config.extraHint ? ' ' + config.extraHint : ''),
                optionalText: '',
                missingMessage: _('This value is required.'),
                invalidMessage: invalidMessage,
                validator: function (value, constraints) {
                    if (this.disabled || !this.required && value === '') {
                        return true;
                    }
                    if (this.required && value === '') {
                        this.invalidMessage = _('This value is required.');
                        return false;
                    }
                    if (isNaN(value)) {
                        this.invalidMessage = _('Please enter a numeric value.');
                        return false;
                    }
                    if (this.required && (value < constraints.min || value > constraints.max)) {
                        this.invalidMessage = _('The value must not exceed {0}.').replace('{0}', constraints.max);
                        return false;
                    }
                    if (!value.match(this.pattern)) {
                        this.invalidMessage = _('Please enter a numeric value.');
                        return false;
                    }
                    return true;
                }
            });
        }

        function formatStorageLabel(label, limit) {
            var gigabyte = 1073741824;
            var usageUnit = uomConverter.convertByteToMajorityUnit(gigabyte);
            var unit = uomConverter.getUnitName(usageUnit.unit, {}, {}, this);
            if (limit === false) {
                return lang.replace(label, {unit: ''})
            }
            return limit !== null ? lang.replace(label, {unit: '(' + unit + ')'}) : lang.replace(label, {unit: _('Usage')});
        }

        function canShowQuotas(userQuota, account) {
            if (userQuota != common.USER_QUOTA.OFF) {
                return true;
            }

            for (let counterName of common.getFullCountersList()) {
                var userOfferingItem = common.getUserOfferingItem(account, counterName);
                if (userOfferingItem && userOfferingItem.limit !== -1 && userOfferingItem.limit !== null) {
                    return true;
                }
            }

            return false;
        }

        function getQuotaDescription(userQuota) {
            switch (Number(userQuota)) {
                case common.USER_QUOTA.OFF:
                    return _('Quota management for service users is disabled.');
                case common.USER_QUOTA.STRICT:
                case common.USER_QUOTA.SOFT:
                    return _('Specify the quotas for the service user.');
                default:
                    return '';
            }
        }

        return {
            getQuotaDescription: getQuotaDescription,
            canShowQuotas: canShowQuotas,
            formatStorageLabel: formatStorageLabel,
            getQuotaControls: getQuotaControls
        };
    });


