<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\CloudApi;

use Acronis\Cloud\Client\ApiException;
use Acronis\Cloud\Client\HttpException;
use Acronis\Cloud\Client\Model\AccessPolicies\AccessPolicy;
use Acronis\Cloud\Client\Model\Applications\Application;
use Acronis\Cloud\Client\Model\Infra\Infra;
use Acronis\Cloud\Client\Model\OfferingItems\OfferingItemOutput;
use Acronis\Cloud\Client\Model\Reports\Report;
use Acronis\Cloud\Client\Model\Reports\Stored\StoredReportParamsItems;
use Acronis\Cloud\Client\Model\Tenants\OfferingItem;
use Acronis\Cloud\Client\Model\Tenants\Tenant;
use Acronis\Cloud\Client\Model\Usages\UsageOutput;
use Acronis\Cloud\Client\Model\Users\User;
use Acronis\Cloud\Client\Model\Users\UserPost;
use Acronis\Cloud\Client\Model\Users\UserPut;
use Service_MspApiClient_Request_Struct_CreateGroupInterface;

interface ApiInterface
{
    const NULL_UUID = '00000000-0000-0000-0000-000000000000';

    const EDITION_ANY = '*';
    const EDITION_STD = 'standard';

    /**
     * @var string
     * @deprecated
     */
    const APPLICATION_TYPE_BACKUP = 'backup';

    const OFFERING_ITEM_MAILBOX_INSTANCE_SIZE = 'mailbox_instance_size';
    const OFFERING_ITEM_O365_MAILBOXES = 'o365_mailboxes';
    const OFFERING_ITEM_O365_ONEDRIVE = 'o365_onedrive';
    const OFFERING_ITEM_O365_SHAREPOINT_SITES = 'o365_sharepoint_sites';

    const OFFERING_ITEM_STATUS_ACTIVE = 1;
    const OFFERING_ITEM_STATUS_INACTIVE = 0;

    const UNLIMITED_OFFERING_ITEM_VALUE = null;

    /**
     * @return string
     */
    public function getUrl();

    /**
     * @return string
     */
    public function getLogin();

    /**
     * @return string
     */
    public function getPassword();

    /* ====== Users API ====== */

    /**
     * @return User
     */
    public function getMe();

    /**
     * Returns True if the login exists otherwise False
     *
     * @param string $login
     * @return bool
     */
    public function checkLogin($login);

    /* ====== Methods to manage users ====== */

    /**
     * @param UserPost $userBody
     * @return User
     */
    public function createUser(UserPost $userBody);

    /**
     * @param string $userId
     * @param int $version
     * @return User
     */
    public function deleteUser($userId, $version);

    /**
     * @param string $userId
     * @return User
     */
    public function getUser($userId);

    /**
     * @param string $userId
     * @param UserPut $userBody
     * @return User
     */
    public function updateUser($userId, UserPut $userBody);

    /**
     * @param $userId
     * @param AccessPolicy[] $accessPolicies
     * @return AccessPolicy[]
     */
    public function updateUserAccessPolicies($userId, array $accessPolicies);

    /* ====== Methods to manage tenants ====== */

    /**
     * @param Service_MspApiClient_Request_Struct_CreateGroupInterface $groupInfo
     * @param $parentGroupId
     * @return User
     */
    public function createTenant($groupInfo, $parentGroupId);

    /**
     * @param string $tenantId
     * @return Tenant
     */
    public function getTenant($tenantId);

    /**
     * @param $tenantId
     * @param $putTenant
     * @return Tenant
     */
    public function updateTenant($tenantId, $putTenant);

    /**
     * @param \Service_MspApiClient_Request_Struct_UpdateGroupInterface $groupInfo
     * @param string $tenantId
     * @param integer $version
     * @param string $groupKind
     * @param \Service_MspApiClient_ApiInterface $mspApi
     * @return integer
     */
    public function updateTenantFromGroupInfo($groupInfo, $tenantId, $version, $groupKind, $mspApi);

    /**
     * @param $rootTenantId
     * @param $leafTenantId
     * @return string[] a list of UUIDs
     */
    public function resolveTenantPath($rootTenantId, $leafTenantId);

    /**
     * @param string $tenantId
     * @return UsageOutput[]
     */
    public function getTenantUsages($tenantId);

    /**
     * @param string $tenantId
     * @return User[]
     * @throws ApiException
     * @throws HttpException
     */
    public function getTenantUsers($tenantId);

    /* ====== Methods to manage tenant's applications ====== */

    /**
     * @return Application[]
     */
    public function getApplications();

    /**
     * @param string $type
     * @return Application|null
     */
    public function getApplicationByType($type);

    /**
     * @param string $tenantId
     * @return string[] a list of UUIDs
     */
    public function getTenantApplicationsUuids($tenantId);

    /**
     * @param $tenantId
     * @param $kind
     * @param string $edition
     * @return mixed
     */
    public function getOfferingItemsAvailableForChild($tenantId, $kind, $edition = self::EDITION_ANY);

    /**
     * @param string $tenantId
     * @param string $applicationId
     * @return bool
     */
    public function isApplicationEnabledForTenant($tenantId, $applicationId);

    /**
     * @param string $tenantId
     * @param string $applicationId
     */
    public function enableApplicationForTenant($tenantId, $applicationId);

    /**
     * @param string $tenantId
     * @param string $applicationId
     */
    public function disableApplicationForTenant($tenantId, $applicationId);

    /* ====== Methods to manage offering items ====== */

    /**
     * @param string $tenantId
     * @param string $applicationId
     * @param string $edition
     * @return OfferingItemOutput[]
     */
    public function getTenantApplicationOfferingItems($tenantId, $applicationId, $edition = '*');

    /**
     * @param string $tenantId
     * @param OfferingItem[] $offeringItems
     * @return OfferingItemOutput[]
     */
    public function updateTenantApplicationOfferingItems($tenantId, $offeringItems);

    /**
     * Recursive enable an application for a hierarchy of tenants except root tenant.
     *
     * @param string $rootTenantId
     * @param string $leafTenantId
     * @param string $applicationId
     */
    public function enableApplicationForTenantsChain($rootTenantId, $leafTenantId, $applicationId);

    /**
     * Recursive enable specified offering items with unlimited quotas for a hierarchy of tenants except root tenant.
     * Note: Enabled or not specified offering items won't be affected
     *
     * @param string $rootTenantId
     * @param string $leafTenantId
     * @param string $applicationId
     * @param OfferingItem[] $offeringItems
     */
    public function enableApplicationOfferingItemsForTenantsChain(
        $rootTenantId,
        $leafTenantId,
        $applicationId,
        array $offeringItems
    );

    /**
     * @param $applicationType
     * @return Application|null
     */
    public function getApplication($applicationType);

    /**
     * @param $userPersonalTenantId
     * @return OfferingItemOutput[]
     */
    public function getOfferingItemsAvailableForChildHashMap($userPersonalTenantId);

    /**
     * Update specified only offering items for $leafTenantId.
     * Recursive enable offering items for tenant's parent (see method enableUnlimitedOfferingItemsForTenantsChain)
     *
     * @param string $rootTenantId
     * @param string $leafTenantId
     * @param string $applicationId
     * @param OfferingItem[] $offeringItems
     */
    public function updateApplicationOfferingItemsForTenantsChain(
        $rootTenantId,
        $leafTenantId,
        $applicationId,
        array $offeringItems
    );

    /* ====== Methods to manage reports ====== */

    /**
     * @param string $reportId
     * @return StoredReportParamsItems[]
     */
    public function getStoredUsageReports($reportId);

    /**
     * @param string $reportId
     * @param string $storedReportId
     * @param string $destinationFile
     * @return void
     */
    public function downloadStoredUsageReportArchive($reportId, $storedReportId, $destinationFile);

    /**
     * @param string $tenantId
     * @return Report
     */
    public function orderCurrentUsageReportForAccounts($tenantId);

    /* ====== Methods to manage infra objects ====== */

    /**
     * @param string $tenantId
     * @return string[] a list of UUIDs
     */
    public function getTenantInfrasUuids($tenantId);

    /**
     * @param string[] $uuids
     * @return Infra[]
     */
    public function fetchInfras($uuids);

    /**
     * @param string $tenantId
     * @param string $capability
     * @return Infra[]
     */
    public function fetchTenantInfras($tenantId, $capability);

    /**
     * @param $rootTenantId
     * @param $leafTenantId
     * @return string[] a list of UUIDs
     */
    public function getInfrasUuidsForTenantsChain($rootTenantId, $leafTenantId);

    /**
     * @param $rootTenantId
     * @param $leafTenantId
     * @return Infra[]
     */
    public function fetchInfrasForTenantsChain($rootTenantId, $leafTenantId);
}





