<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Console\Command;

use Acronis\App\Services\Redis\PureRedisAwareTrait;
use Acronis\App\Services\Redis\RedisClient;
use Acronis\App\Utils\Func;
use Log;
use Str;
use Symfony\Component\Lock\Factory;
use Symfony\Component\Lock\Store\FlockStore;

/**
 * Implement method getCommandName, getLog or use this trait only in console commands extending Loggable
 * @package Acronis\App\Console\Command
 */
trait LockableTrait
{
    static protected $lockPrefix = 'lock';

    use PureRedisAwareTrait;

    /**
     * @param callable $method
     * @param $arguments
     * @param null $lockBaseName
     * @param $blocking
     * @throws \Exception
     */
    public function executeWithLock(callable $method, $arguments, $lockBaseName = null, $blocking = false)
    {
        $lockBaseName = $lockBaseName ?: Func::getCallerName();
        $lockId = $this->generateLockId($lockBaseName, $arguments);
        $store = new FlockStore(DIR_TMP);
        $factory = new Factory($store);
        Log::getInstance()->notice('Creating lock "%s".', $lockId);
        $lock = $factory->createLock($lockId);
        Log::getInstance()->notice('Lock "%s" was created with TTL 300s (vendor defaults).', $lockId);

        if (!$lock->acquire($blocking)) {
            Log::getInstance()->warning(
                'Unable to acquire lock "%s", another process acquired the lock.', $lockId
            );
            return;
        }

        try {
            $start = microtime(true);
            Log::getInstance()->notice('Lock "%s" was acquired - started execution.', $lockId);
            $method();
        } finally {
            $lock->release();
            Log::getInstance()->notice(
                'Lock id "%s" was released. Execution time: %s.',
                $lockId, round(microtime(true) - $start, 2)
            );
        }
    }

    /**
     * @param $commandName
     * @param array $arguments
     * @return string
     */
    private function generateLockId($lockBaseName, $arguments = [])
    {
        return Str::format(
            '%s_%s|%s',
            static::$lockPrefix, $lockBaseName, implode(',', $arguments)
        );
    }

    public function releaseAllLocks($lockBaseName)
    {
        Log::getInstance()->notice('Releasing all locks starting with "%s".', $lockBaseName);
        $client = new RedisClient($this->getPureRedis());
        $client->deleteAll($this->generateLockId($lockBaseName));
        Log::getInstance()->notice('All locks starting with "%s" were released.', $lockBaseName);
    }
}