<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_BaseClient_RestTransport implements Service_BaseClient_IRestTransport
{
    const LOG_MESSAGE_REQUEST_FORMAT = "--debug Request %s to %s request body: (%s)";
    const LOG_MESSAGE_ERROR_FORMAT = "--debug Response %s from %s error %s";
    const LOG_MESSAGE_RESPONSE_FORMAT = "--debug Response %s from %s code %s resp_body: (%s)";
    const ACRONIS_APS_USERAGENT_MASK = "%s-version-%s";
    const ACRONIS_APS_PACKAGE = 'Acronis-Aps-Package';

    const BAAS_ASYNC_IN_PROGRESS = 'The asynchronous operation still in progress. Try again later.';
    const TRY_NO = 'Attempt #%s...';
    const ERROR_SERVICE_UNAVAILABLE = 'The service is currently not available. Please try again later.';
    const ERROR_REQUEST = 'Request error: %s';

    const HTTP_CODE_SUCCESS = 200;
    const HTTP_CODE_MULTIPLE_CHOICE = 300;
    const HTTP_CODE_BAD_REQUEST = 400;
    const HTTP_CODE_FORBIDDEN = 403;
    const HTTP_CODE_UNKNOWN_STATUS = 409;
    const HTTP_CODE_BAD_REQUEST_HIGH = 499;
    const HTTP_CODE_RETRY = 449;
    const HTTP_CODE_ERROR = 500;
    const HTTP_CODE_ERROR_HIGH = 599;
    const HTTP_CODE_ERROR_TIMEOUT = 0;

    const HTTP_HEADER_CONNECTION_CLOSE = 'Connection: close';

    const RETRIES_COUNT = 10;
    const RETRIES_DELAY = 5; // seconds

    const REQUEST_USAGE_REQUEST_PART = 'reports/ordered';

    private $_cookieFile;

    /** @var Service_BaseClient_TransportStrategy_ITransportStrategy */
    private $_sendingStrategy;
    /** @var CurlInterface */
    private $_curl;
    /** @var BuildInfoInterface */
    private $_buildInfo;
    /** @var Time */
    private $_time;
    /** @var bool */
    private $_verifySsl = false;

    /**
     * @param CurlInterface $curl
     * @param BuildInfoInterface $buildInfo
     * @param Time $time
     */
    public function __construct(CurlInterface $curl = NULL, BuildInfoInterface $buildInfo = NULL, Time $time = NULL)
    {
        $this->_curl = is_null($curl) ? new Curl() : $curl;
        $this->_buildInfo = is_null($buildInfo) ? new BuildInfo() : $buildInfo;
        $this->_time = is_null($time) ? new Time() : $time;
    }

    public function refresh()
    {
        $this->_curl->refresh();
    }

    /**
     * @param string $cookieFile
     */
    public function setCookieFile($cookieFile)
    {
        $this->_cookieFile = $cookieFile;
    }

    /**
     * @param bool $verifySsl
     */
    public function setVerifySsl($verifySsl)
    {
        $this->_verifySsl = $verifySsl;
    }

    /**
     * @return bool
     */
    public function hasCookieFile()
    {
        return !is_null($this->_cookieFile);
    }

    public function invalidateCookies()
    {
        if (!is_null($this->_cookieFile) && file_exists($this->_cookieFile))
        {
            unlink($this->_cookieFile);
        }
    }

    /**
     * @param Service_BaseClient_TransportStrategy_ITransportStrategy $sendingStrategy
     */
    public function setSendingStrategy(Service_BaseClient_TransportStrategy_ITransportStrategy $sendingStrategy)
    {
        $this->_sendingStrategy = $sendingStrategy;
    }

    /**
     * @return Service_BaseClient_TransportStrategy_ITransportStrategy
     */
    public function getSendingStrategy()
    {
        return $this->_sendingStrategy;
    }

    /**
     * @param string $uri
     * @param string $method
     * @param string $requestBody
     * @param string $authHeaders
     * @throws Exception
     * @return array(code, responseBody)
     */
    public function execute($uri, $method, $requestBody, $authHeaders = NULL)
    {
        Log::getInstance()->event(
            sprintf(
                self::LOG_MESSAGE_REQUEST_FORMAT,
                $method,
                $uri,
                $this->_getJson($requestBody)
            )
        );

        $userAgent = sprintf(self::ACRONIS_APS_USERAGENT_MASK,
                self::ACRONIS_APS_PACKAGE,
                $this->_buildInfo->getPackageVersion());

        // close the connection to not keep a lot open sockets
        $headers = $authHeaders ?: array();
        $headers []= self::HTTP_HEADER_CONNECTION_CLOSE;

        $this->_sendingStrategy->prepare($this->_curl, $requestBody, $headers);

        $this->_curl->setOption(CURLOPT_URL, $uri);
        $this->_curl->setOption(CURLOPT_CUSTOMREQUEST, $method);
        $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);

        if (is_string($this->_cookieFile)) {
            $this->_curl->setOption(CURLOPT_COOKIEJAR, $this->_cookieFile);
        }

        $this->_curl->setOption(CURLOPT_COOKIEFILE, $this->_cookieFile);

        $this->_curl->setOption(CURLOPT_FOLLOWLOCATION, true);
        $this->_curl->setOption(CURLOPT_USERAGENT, $userAgent);

        if (!$this->_verifySsl) {
            $this->_curl->setOption(CURLOPT_SSL_VERIFYPEER, false);
            $this->_curl->setOption(CURLOPT_SSL_VERIFYHOST, false);
        }

        $tries = 0;
        do
        {
            $responseBody = $this->_curl->execute();
            $errorNo = $this->_curl->getErrorNo();
            if ($errorNo)
            {
                $error = sprintf(self::ERROR_REQUEST, $this->_curl->getError());
                Log::getInstance()->error(sprintf(self::LOG_MESSAGE_ERROR_FORMAT, $method, $uri, $error));
                throw new Service_BaseClient_IOException($error, $errorNo);
            }
            $httpCode = $this->_curl->getInfo(CURLINFO_HTTP_CODE);

            if ($tries)
            {
                Log::getInstance()->event(sprintf(self::TRY_NO, $tries));
            }
            if (strpos($uri, self::REQUEST_USAGE_REQUEST_PART) === false) {
                Log::getInstance()->event(sprintf(self::LOG_MESSAGE_RESPONSE_FORMAT, $method, $uri, $httpCode, $responseBody));
            }
        }
        while ($this->_checkHttpCode($httpCode, $tries));

        $responseHeaders = $this->_curl->getHeaders();

        if (self::HTTP_CODE_RETRY === $httpCode)
        {
            throw new Service_BaseClient_HTTPException(self::BAAS_ASYNC_IN_PROGRESS, $httpCode, NULL, $responseHeaders);
        }
        if ($httpCode >= self::HTTP_CODE_BAD_REQUEST && $httpCode <= self::HTTP_CODE_ERROR_HIGH)
        {
            throw new Service_BaseClient_HTTPException(self::ERROR_SERVICE_UNAVAILABLE, $httpCode, NULL, $responseHeaders);
        }

        return array($httpCode, $responseBody);
    }

    /**
     * @param string $requestBody
     * @return string
     */
    private function _getJson($requestBody)
    {
        return is_array($requestBody) ? json_encode($requestBody) : $requestBody;
    }

    /**
     * @param int $httpCode
     * @param int $tries
     * @return bool
     */
    private function _checkHttpCode($httpCode, &$tries)
    {
        $result = self::HTTP_CODE_RETRY === $httpCode
            && $tries < self::RETRIES_COUNT
            && $this->_time->sleep(self::RETRIES_DELAY) !== false;
        $tries++;
        return $result;
    }
}
