<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require_once(__DIR__ . '/app/bootstrap.php');

/**
 * @type("http://acronis.com/aps/cloud-backup-v1/backup-profile/5.1")
 * @implements("http://aps-standard.org/types/core/resource/1.0")
 */
class profile extends APSResource
{
    const ROOT_SUBSCRIPTION_ID = 1;

    const EX_MSG_ALREADY_BOUND = 'Service profile already bound with resource type';
    const EX_MSG_NOT_BOUND = 'Service profile not bound with resource type';
    const EX_MSG_CREATING_PROFILE_TASK = 'Creating profile (%s)';

    const LOG_OPEN_API_EXCEPTION_SKIPPED = 'Skip exception "%s"';
    const LOG_CALLED_PROVISION = 'Profile (%s) provision was called.';
    const LOG_CALLED_CONFIGURE = 'Profile (%s) configure was called.';
    const LOG_CALLED_PROVISION_ASYNC = 'Asynchronous provision for profile ($s) was called.';
    const LOG_CALLED_BIND_RESOURCE_TYPE = 'Bind resource type with service profile %s was called.';
    const LOG_CALLED_UNBIND_RESOURCE_TYPE = 'Unbind resource type with service profile %s was called.';
    const LOG_CALLED_UNPROVISION = 'Unprovision resource %s was called.';
    const LOG_CALLED_GET_PARTNER_CREDENTIALS = 'Get partner %s credentials was called.';
    const LOG_CALLED_IS_USED = 'Profile get status %s was called.';
    const LOG_CALLED_UPGRADE = 'Profile (%s) upgrade was called';
    const LOG_WARNING_UNABLE_TO_DETERMINE_DATACENTER = "Unable to determine the datacenter. Count: %s";
    const FIELD_IS_USED = 'isUsed';

    const USER_QUOTA_OFF = 0;
    const USER_QUOTA_STRICT = 1;
    const USER_QUOTA_SOFT = 2;

    const STORAGE_POLICY_PARENT = 0;
    const STORAGE_POLICY_OWN = 1;

    const STORAGE_DEFAULT_ID = 'default';
    const STORAGE_LOCATION_LOCAL_AND_CLOUD = 1;

    const MSP_GROUP_TYPE_EUC = 0;
    const MSP_GROUP_TYPE_RESELLER = 1;

    /* ======= RESOURCE PROPERTIES ======= */

    /**
     * @link("http://acronis.com/aps/cloud-backup-v1/globals/9.0")
     * @required
     */
    public $global;

    /**
     * @link("http://acronis.com/aps/cloud-backup-v1/tenant/9.0[]")
     */
    public $tenants;

    /**
     * @type("string")
     */
    public $name;

    /**
     * @type(string[])
     */
    public $notifications;

    /**
     * @type("integer")
     */
    public $backupNotification;

    /**
     * @type("integer")
     */
    public $managementNotification;

    /**
     * @type("integer")
     */
    public $storagePolicy;

    /**
     * @type(BackupProfileStorage[])
     */
    public $storages;

    /**
     * @type(BackupProfileDatacenter[])
     */
    public $datacenters;

    /**
     * @type("integer")
     */
    public $userQuota;

    /**
     * @deprecated
     * @type("integer")
     */
    public $storageLocation;

    /**
     * @type("integer")
     * @readonly
     */
    public $resourceTypeId;

    /**
     * @type("integer")
     */
    public $freemiumMailboxStorage = 0;

    /**
     * 0 - Customer account (=Customer group in ABC) DEFAULT
     * 1 - Partner account (=Partner group in ABC)
     *
     * @type("integer")
     */
    public $mspGroupType = 0;

    /* ======= OTHER PROPERTIES ======= */

    /* ======= RESOURCE METHODS ======= */

    public function provision()
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_PROVISION, $this->aps->id));
        throw new \Rest\Accepted(
            $this,
            sprintf(self::EX_MSG_CREATING_PROFILE_TASK, $this->aps->id),
            self::ASYNC_OPERATION_TIMEOUT
        );
    }

    public function provisionAsync()
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_PROVISION_ASYNC, $this->aps->id));
        $this->_bindResourceType();
    }

    public function configure($resource)
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_PROVISION, $this->aps->id));
        parent::configure($resource);
        if ($this->storagePolicy == self::STORAGE_POLICY_OWN) {
            $this->storages = array();
        }
    }

    public function unprovision()
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_UNPROVISION, $this->id()));
        if (!is_null($this->resourceTypeId)) {
            $this->_unbindResourceType();
        }
    }

    public function upgrade()
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_UPGRADE, $this->aps->id));

        if ($this->aps->type === self::CLOUD_BACKUP_PROFILE_TYPE) {
            return;
        }

        $this->aps->type = self::CLOUD_BACKUP_PROFILE_TYPE;
        if (is_null($this->userQuota)) {
            $this->userQuota = self::USER_QUOTA_STRICT;
        }
        if (is_null($this->storagePolicy)) {
            $this->storagePolicy = self::STORAGE_POLICY_PARENT;
        }
        if (is_null($this->freemiumMailboxStorage)) {
            $this->freemiumMailboxStorage = 0;
        }
        if (is_null($this->mspGroupType)) {
            $this->mspGroupType = self::MSP_GROUP_TYPE_EUC;
        }


        $this->notifications = Model_PropertyConverters_NotificationConverter::getNotifications(
            Arr::get($this, 'backupNotification', 0),
            Arr::get($this, 'managementNotification', false)
        );

        $this->_upgradeStorages();
        if (isset($this->storageId)) {
            unset($this->storageId);
        }
        $this->update();
    }

    /* ======= RESOURCE EVENTS ======= */

    /* ======= RESOURCE OPERATIONS ======= */

    /**
     * @verb(POST)
     * @path("/bind_resource_type")
     *
     * @param()
     */
    public function bindResourceType()
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_BIND_RESOURCE_TYPE, $this->id()));
        $this->_bindResourceType();
        $this->update();
        throw new Rest\RestException(200, "No Content");
    }

    /**
     * @verb(POST)
     * @path("/unbind_resource_type")
     *
     * @param()
     */
    public function unbindResourceType()
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_UNBIND_RESOURCE_TYPE, $this->id()));
        $this->_unbindResourceType();
        $this->update();
        throw new Rest\RestException(200, "No Content");
    }

    /**
     * @verb(GET)
     * @path("/get_resource_type")
     *
     * @param()
     */
    public function getResourceType()
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_UNBIND_RESOURCE_TYPE, $this->id()));
        return $this->_getResourceType();
    }

    /**
     * @verb(GET)
     * @path("/get_status")
     *
     * @param()
     */
    public function getStatus()
    {
        Log::getInstance()->event(sprintf(self::LOG_CALLED_IS_USED, $this->id()));

        return array(
            self::FIELD_IS_USED => count($this->tenants) > 0
        );
    }

    /* ======= OTHER METHODS ======= */

    /**
     * @return PartnerCredentials
     * @throws Exception
     */
    public function getPartnerCredentials()
    {
        Log::getInstance()->event(self::LOG_CALLED_GET_PARTNER_CREDENTIALS);
        $message = sprintf(self::EXCEPTION_METHOD_IS_NOT_APPLICABLE_MESSAGE, __METHOD__);
        Log::getInstance()->warning($message);
        throw new Exception($message);
    }

    private function _bindResourceType()
    {
        if (isset($this->resourceTypeId)) {
            throw new Exception(self::EX_MSG_ALREADY_BOUND);
        }

        /** @var Service_OpenApi_Api $openApi */
        $openApi = $this->_getOpenApi();

        $applicationId = $openApi->apsGetApplicationId(self::PACKAGE_ID);
        $resourceId = $this->id();

        $this->resourceTypeId = $openApi->addRTServiceLink($this->name, $applicationId, $resourceId, '', null);
    }

    private function _unbindResourceType()
    {
        if (!isset($this->resourceTypeId)) {
            throw new Exception(self::EX_MSG_NOT_BOUND);
        }

        /** @var Service_OpenApi_Api $openApi */
        $openApi = $this->_getOpenApi();

        try {
            $openApi->removeResourceType($this->resourceTypeId);
        } catch (Service_OpenApi_Validator_InvalidStatusException $e) {
            $code = $e->getCode();
            if ($code != Service_OpenApi_Validator_InvalidStatusException::EX_ERR_NOT_FOUND) {
                throw $e;
            }
            Log::getInstance()->event(sprintf(self::LOG_OPEN_API_EXCEPTION_SKIPPED, $e->getMessage()));
        }

        $this->resourceTypeId = null;
    }

    private function _getResourceType()
    {
        if (!isset($this->resourceTypeId)) {
            return NULL;
        }

        /** @var Service_OpenApi_Api $openApi */
        $openApi = $this->_getOpenApi();

        /**
         * @var Service_OpenApi_Response_SubscriptionInfo $subscription
         */
        $subscription = $openApi->getSubscription(self::ROOT_SUBSCRIPTION_ID);
        $resources = $subscription->getResourceTypes();
        /**
         * @var Service_OpenApi_Response_SubscriptionInfo_ResourceType $resource
         */
        foreach ($resources as $resource) {
            $resourceTypeId = $resource->getResourceTypeId();
            if ($resourceTypeId == $this->resourceTypeId) {
                return array(
                    Service_OpenApi_Api::FIELD_RESOURCE_TYPE_ID => $resourceTypeId,
                    Service_OpenApi_Api::FIELD_NAME => $resource->getName()
                );
            }
        }

        return NULL;
    }

    private function _upgradeStorages()
    {
        if (!empty($this->storages) || $this->storagePolicy != self::STORAGE_POLICY_PARENT || is_null($this->storageId) || $this->storageId === self::STORAGE_DEFAULT_ID) {
            return;
        }

        $storage = new BackupProfileStorage();
        $storage->datacenterId = globals::DEFAULT_DATACENTER_ID;
        $storage->storageId = $this->storageId;

        $this->storages = array($storage);
    }
}
